module detector
(
	iReset,
	
	iUserReset,
	
	iClock,
	oLED,
	
	oXCoord,
	oYCoord,
	
	//the images from the SDRAM
	oSDRAM_READ_CLOCK,  //data clock on the sdram (each cycle gets a new word)
	oSDRAM_READ_LOGIC,  //set high when data is wanted
	iSDRAM_IMAGE1,		   //data from image 1
	iSDRAM_IMAGE1_EMPTY,
	iSDRAM_IMAGE2,		   //data from image 2
	iCCDFRAMECOUNT,
	iCCDCLOCK,
	//the output to the VGA
	oVGASRAM_ADDR,	//address of VGA output SRAM (320x240)
	oVGASRAM_DATA,	//data for		 "			"
	iVGA_OK_TO_WRITE,		//signal indicating it is OK to write to the SRAM	
	iVGA_CTRL_CLK
);
	input   		iReset;
	input			iUserReset;
	input 			iClock;
	output [17:0]	oLED;
	
	output [9:0]	oXCoord;
	output [9:0]	oYCoord;
	
	//the images from the SDRAM
	output 			oSDRAM_READ_CLOCK;  	//data clock on the sdram (each cycle gets a new word)
	output 			oSDRAM_READ_LOGIC; 		//set high when data is wanted
	input [15:0]	iSDRAM_IMAGE1;		   	//data from image 1
	input 			iSDRAM_IMAGE1_EMPTY;	// indicates the FIFO buffer is empty
	input [15:0]	iSDRAM_IMAGE2;		   	//data from image 2
	input [31:0]   	iCCDFRAMECOUNT;         // the framecount
	input			iCCDCLOCK;
	//the output to the VGA
	output [17:0]	oVGASRAM_ADDR;		//address of VGA output SRAM (320x240)
	output [15:0]	oVGASRAM_DATA;		//data for		 "			"
	input 			iVGA_OK_TO_WRITE;		//signal indicating it is OK to write to the SRAM	
	input			iVGA_CTRL_CLK;

//---------------------------------------------------------------------------------------------------------------------------

	reg [9:0] Xstart, Xend, Ystart, Yend;	
	
	reg [9:0] ImXcoord, ImXcoord2;	//the coordinates beeing read out of the sdram based images
	reg [9:0] ImYcoord, ImYcoord2;
		
	reg [17:0] rLED;

	reg [17:0] rVGASRAM_ADDR;
	reg [15:0] rVGASRAM_DATA;
	
	reg			flag;
	reg [7:0]   sum, sum1, sum2, sum3, sum4, sum5, sum6, sum7, sum8, sum9;
											
	assign oSDRAM_READ_LOGIC = iVGA_OK_TO_WRITE;
	assign oSDRAM_READ_CLOCK = iVGA_CTRL_CLK;
	assign oVGASRAM_ADDR = rVGASRAM_ADDR;
	assign oVGASRAM_DATA = rVGASRAM_DATA;		
	assign oLED = rLED;
	
	assign oXCoord = ImXcoord;
	assign oYCoord = ImYcoord;
	
	reg detect;

	always @ (posedge oSDRAM_READ_CLOCK)
	begin
		//the ram is a FIFO buffer, so we will need to scan and selectively add the correct pixels.
		if ((iReset)||(iUserReset))	
		begin
			ImXcoord <= 10'd0;
			ImYcoord <= 10'd0;
			rVGASRAM_ADDR <= 18'd0;
			
			//initial box coords
			//Xstart <= 10'd303;
			//Xend <= 10'd335;
			//Ystart <= 10'd203;
			//Yend <= 10'd235;
			
			Xstart <= 10'd273;
			Xend <= 10'd365;
			Ystart <= 10'd173;
			Yend <= 10'd265;
			
			sum  <= 8'd0;
			sum1 <= 8'd0;
			sum2 <= 8'd0;
			sum3 <= 8'd0;
			sum4 <= 8'd0;
			sum5 <= 8'd0;
			sum6 <= 8'd0;
			sum7 <= 8'd0;
			sum8 <= 8'd0;
			sum9 <= 8'd0;
			
			flag <= 0;
			detect <= 1'b0;
			
			rLED <= 18'b100000000000000000;
		end
		
		else if (iVGA_OK_TO_WRITE)  //modify display during sync
		begin
			
			case (detect)
			
			1'b0:
			begin		

			rVGASRAM_ADDR <= {ImXcoord[9:1], ImYcoord[9:1]};
			
			//Green Box for tracking - top and bottom
			if ((ImXcoord >= Xstart)  	&& 
				(ImXcoord <= Xend)		&&
				((ImYcoord == Ystart) || (ImYcoord == Yend)))
				begin			
					if (flag==0) rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b010000}; 
					else rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b001000};
				end
				
			//Green Box for tracking - left and right
			else if ((ImYcoord >= Ystart)  	&& 
					 (ImYcoord <= Yend)		&&
					 ((ImXcoord == Xstart) || (ImXcoord == Xend)))				
				begin
					if (flag==0) rVGASRAM_DATA <= {iSDRAM_IMAGE1[9:0], 6'b010000}; 
					else rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b001000};
				end					
			
			//rest of image																							
			else if (ImYcoord < 479)	rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b000000}; 			
			
			//black to all other
			else
				rVGASRAM_DATA <=  16'h0;
			
			//color in red light that is above threshold
			if (((ImYcoord > Ystart) && (ImYcoord < Yend) && (ImXcoord > Xstart) && (ImXcoord < Xend)) &&
				(iSDRAM_IMAGE1[9:0] >= 700))
			begin
				rVGASRAM_DATA <= {iSDRAM_IMAGE1[9:0],6'b100000};
			end
			
			rLED <= 18'b010000000000000000;
			
			end
			
			1'b1:
			begin
			if (((ImYcoord > Ystart) && (ImYcoord < Yend) && (ImXcoord > Xstart) && (ImXcoord < Xend)) &&
					(iSDRAM_IMAGE1[9:0] >= 700))
				begin
					sum <= sum + 8'd1;
					
					if ((ImYcoord > Ystart) && (ImYcoord < (Ystart+30)) && (ImXcoord > Xstart) && (ImXcoord < (Xstart+30)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum1 <= sum1 + 8'd1;
					end
					else if ((ImYcoord > Ystart) && (ImYcoord < (Ystart+30)) && (ImXcoord > (Xstart+30)) && (ImXcoord < (Xstart+60)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum2 <= sum2 + 8'd1;
					end
					else if ((ImYcoord > Ystart) && (ImYcoord < (Ystart+30)) && (ImXcoord > (Xstart+60)) && (ImXcoord < (Xstart+90)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum3 <= sum3 + 8'd1;
					end
					else if ((ImYcoord > (Ystart+30)) && (ImYcoord < (Ystart+60)) && (ImXcoord > Xstart) && (ImXcoord < (Xstart+30)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum4 <= sum4 + 8'd1;
					end
					else if ((ImYcoord > (Ystart+30)) && (ImYcoord < (Ystart+60)) && (ImXcoord > (Xstart+30)) && (ImXcoord < (Xstart+60)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum5 <= sum5 + 8'd1;
					end
					else if ((ImYcoord > (Ystart+30)) && (ImYcoord < (Ystart+60)) && (ImXcoord > (Xstart+60)) && (ImXcoord < (Xstart+90)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum6 <= sum6 + 8'd1;
					end
					else if ((ImYcoord > (Ystart+60)) && (ImYcoord < (Ystart+90)) && (ImXcoord > Xstart) && (ImXcoord < (Xstart+30)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum7 <= sum7 + 8'd1;
					end
					else if ((ImYcoord > (Ystart+60)) && (ImYcoord < (Ystart+90)) && (ImXcoord > (Xstart+30)) && (ImXcoord < (Xstart+60)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum8 <= sum8 + 8'd1;
					end
					else if ((ImYcoord > (Ystart+60)) && (ImYcoord < (Ystart+90)) && (ImXcoord > (Xstart+60)) && (ImXcoord < (Xstart+90)) && (iSDRAM_IMAGE1[9:0] >= 700))
					begin
						sum9 <= sum9 + 8'd1;
					end
				end	
				
				rLED <= 18'b001000000000000000;
				
			end
			
			endcase
			
			if (ImXcoord == 639)
			begin
				ImXcoord <= 0;
				if (ImYcoord == 479)				
				begin
					ImYcoord <= 0;
					sum <= 8'd0;
					sum1 <= 8'd0;
					sum2 <= 8'd0;
					sum3 <= 8'd0;
					sum4 <= 8'd0;
					sum5 <= 8'd0;
					sum6 <= 8'd0;
					sum7 <= 8'd0;
					sum8 <= 8'd0;
					sum9 <= 8'd0;
					
					if (detect==1'b0) detect <= 1'b1;
					else detect <= 1'b0;	
					if (sum > 50) flag <= 1;
					else flag <= 0;
					if (sum<500)
					begin
					if ((sum1>sum2)&&(sum1>sum3)&&(sum1>sum4)&&(sum1>sum5)&&(sum1>sum6)&&(sum1>sum7)&&(sum1>sum8)&&(sum1>sum9)&&(sum1 > 20))
						begin
							if (Xstart<10'd34) Xstart <= 10'd1;
							else Xstart <= Xstart - 10'd34;
							if (Xstart<10'd34) Xend <= 10'd93;
							else Xend   <= Xend   - 10'd34;
							if (Ystart<10'd34) Ystart <= 10'd1;
							else Ystart <= Ystart - 10'd34;
							if (Ystart<10'd34) Yend <= 10'd93;
							else Yend   <= Yend   - 10'd34;
						end
					else if ((sum2>sum1)&&(sum2>sum3)&&(sum2>sum4)&&(sum2>sum5)&&(sum2>sum6)&&(sum2>sum7)&&(sum2>sum8)&&(sum2>sum9)&&(sum2 > 20))
						begin
							if (Ystart<10'd34) Ystart <= 10'd1;
							else Ystart <= Ystart - 10'd34;
							if (Ystart<10'd34) Yend <= 10'd93;
							else Yend   <= Yend   - 10'd34;
						end
					else if ((sum3>sum2)&&(sum3>sum1)&&(sum3>sum4)&&(sum3>sum5)&&(sum3>sum6)&&(sum3>sum7)&&(sum3>sum8)&&(sum3>sum9)&&(sum3 > 20))
						begin
							if ((Xend+10'd34)>=639) Xstart <= 10'd547;
							else Xstart <= Xstart + 10'd34;
							if ((Xend+10'd34)>=639) Xend <= 10'd639;
							else Xend   <= Xend   + 10'd34;
							if (Ystart<10'd34) Ystart <= 10'd1;
							else Ystart <= Ystart - 10'd34;
							if (Ystart<10'd34) Yend <= 10'd93;
							else Yend   <= Yend   - 10'd34;
						end
					else if ((sum4>sum2)&&(sum4>sum3)&&(sum4>sum1)&&(sum4>sum5)&&(sum4>sum6)&&(sum4>sum7)&&(sum4>sum8)&&(sum4>sum9)&&(sum4 > 20))
						begin
							if (Xstart<10'd34) Xstart <= 10'd1;
							else Xstart <= Xstart - 10'd34;
							if (Xstart<10'd34) Xend <= 10'd93;
							else Xend   <= Xend   - 10'd34;
						end
					//it's in the middle so do nothing
					/*else if ((sum5>sum2)&&(sum5>sum3)&&(sum5>sum4)&&(sum5>sum1)&&(sum5>sum6)&&(sum5>sum7)&&(sum5>sum8)&&(sum5>sum9))
						begin
							;
						end*/
					else if ((sum6>sum2)&&(sum6>sum3)&&(sum6>sum4)&&(sum6>sum5)&&(sum6>sum1)&&(sum6>sum7)&&(sum6>sum8)&&(sum6>sum9)&&(sum6 > 20))
						begin
							if ((Xend+10'd34)>=639) Xstart <= 10'd547;
							else Xstart <= Xstart + 10'd34;
							if ((Xend+10'd34)>=639) Xend <= 10'd639;
							else Xend   <= Xend   + 10'd34;
						end
					else if ((sum7>sum2)&&(sum7>sum3)&&(sum7>sum4)&&(sum7>sum5)&&(sum7>sum6)&&(sum7>sum1)&&(sum7>sum8)&&(sum7>sum9)&&(sum7 > 20))
						begin
							if (Xstart<10'd34) Xstart <= 10'd1;
							else Xstart <= Xstart - 10'd34;
							if (Xstart<10'd34) Xend <= 10'd93;
							else Xend   <= Xend   - 10'd34;
							if ((Yend+10'd34)>=479) Ystart <= 10'd387;
							else Ystart <= Ystart + 10'd34;
							if ((Yend+10'd34)>=479) Yend <= 10'd479;
							else Yend   <= Yend   + 10'd34;
						end
					else if ((sum8>sum2)&&(sum8>sum3)&&(sum8>sum4)&&(sum8>sum5)&&(sum8>sum6)&&(sum8>sum7)&&(sum8>sum1)&&(sum8>sum9)&&(sum8 > 20))
						begin
							if ((Yend+10'd34)>=479) Ystart <= 10'd387;
							else Ystart <= Ystart + 10'd34;
							if ((Yend+10'd34)>=479) Yend <= 10'd479;
							else Yend   <= Yend   + 10'd34;
						end
					else if ((sum9>sum2)&&(sum9>sum3)&&(sum9>sum4)&&(sum9>sum5)&&(sum9>sum6)&&(sum9>sum7)&&(sum9>sum8)&&(sum9>sum1)&&(sum9 > 20))
						begin
							if ((Xend+10'd34)>=639) Xstart <= 10'd547;
							else Xstart <= Xstart + 10'd34;
							if ((Xend+10'd34)>=639) Xend <= 10'd639;
							else Xend   <= Xend   + 10'd34;
							if ((Yend+10'd34)>=479) Ystart <= 10'd387;
							else Ystart <= Ystart + 10'd34;
							if ((Yend+10'd34)>=479) Yend <= 10'd479;
							else Yend   <= Yend   + 10'd34;
						end
					end
				end			
				else
				begin
					ImYcoord <= ImYcoord + 1;
				end
			end
			else
				ImXcoord <= ImXcoord+1;
				
		end
		
	end

endmodule
